#ifndef __KVIEWPART_H
#define __KVIEWPART_H


#include <kparts/part.h>
#include <kparts/browserextension.h>
#include <kparts/factory.h>
#include <kdirwatch.h>
#include <qtimer.h>

class KAboutDialog;
class KAccel;
class KAction;
class KSelectAction;
class KTempFile;
class KToggleAction;
class KViewPartExtension;
class KInstance;
class QSize;
class pageSizeDialog;

#include "kviewpart_iface.h"
#include "kmultipage.h"
#include "pageSize.h"
#include "zoom.h"
#include "zoomlimits.h"

class KViewPartFactory : public KParts::Factory
{
  Q_OBJECT

public:
  KViewPartFactory();
  virtual ~KViewPartFactory();

  virtual KParts::Part *createPartObject( QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name, const char *classname, const QStringList &args );
  
  static KInstance *instance();

private:
  static KInstance *s_instance;
};


class KViewPart : public KViewPart_Iface
{
  Q_OBJECT

public:
  KViewPart(QString partname, QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name);
  virtual ~KViewPart();

  bool isValid() { return multiPage; }

  // Returns a description of the current page size, for use in the
  // statusbar of the kviewshell that embeds this KViewPart.
  QString pageSizeDescription(void);

signals:
  void zoomChanged(const QString &);
  void pageChanged(const QString &);
  void sizeChanged(const QString &);
  void scrollbarStatusChanged(bool);

public slots:
  void slotSetFullPage(bool fullpage);
  virtual bool closeURL();
  virtual QStringList fileFormats();
  void setStatusBarTextFromMultiPage(const QString &);
  /** Calling this slot will cause the kmultipage to reload the file */
  void reload(void);

protected slots:
  void slotShowSidebar();
  void slotMedia (int);

  void goToPage();

  void zoomIn();
  void zoomOut();

  void disableZoomFit();
  void updateZoomLevel();

  void enableFitToPage(bool);
  void enableFitToHeight(bool);
  void enableFitToWidth(bool);

  void fitToPage();
  void fitToHeight();
  void fitToWidth();

  void slotPrint();

  void fileChanged(const QString&);

  // Connected to the QLineEdit in the toolbar.
  void setZoomValue(const QString &);


protected:
  KToggleAction *showSidebar, *scrollbarHandling;
  KSelectAction *orientation, *media, *zoom_action;

  virtual bool openFile();

  void connectNotify ( const char * );

  void partActivateEvent( KParts::PartActivateEvent *ev );
  void guiActivateEvent( KParts::GUIActivateEvent *ev );

private slots:
  void pageInfo(int numpages, int currentpage);
  void checkActions();

  /** This is the slot where mouseWheel events are processed that come
      from the multipage/scrollview. This method calles scroll, a
      delta-value of 120 (i.e. one notch on a standard wheel mouse)
      scrolls by two 'lines'. */
  void wheelEvent(QWheelEvent *);

  // This slot is connected to userRequestedPaperSize. Whenever the
  // size changes, this slot is called. It checks if the
  // userRequestedPaperSize has an orientation and
  // enables/set/disables the menu entries accordingly. It also sets
  // the media menu entries. Finally, the slot sets the _paperSize, if
  // appropriate, and initiates an update of the GUI, if necessary.
  void slotUserPreferredSizeChanged(simplePageSize siz);

  // This slot is connected to the useDocumentSpecifiedSize action,
  // and is called when the user checks/unchecks the "use document
  // specified paper size" menu entry
  void slotUseDocumentSpecifiedSize(bool);

  // This slot is connected to _paperSize: whenever the paper size
  // changes, this slot is called and the necessary upates in the GUI
  // are performed
  void setPaperSize(simplePageSize size);

  // Reacts to page size requests of the multipage
  void slotPageSizeRequests(const pageSize &size);

  void slotStartFitTimer();

  void doSettings();
  void preferencesChanged();

  void aboutKViewShell();

private:
  // This method reads the configuration file. It should only be
  // called when no document is loaded.
  void readSettings();
  void writeSettings();
  
  // The method openFile of this kviewpart can be called even if
  // m_file points to a compressed file. In that case, the temporary
  // file tmpUnzipped will be created and a decompressed copy of the
  // file stored there.
  KTempFile *tmpUnzipped;

  KDirWatch *watch;
  QString _partname;
  KAccel *accel;
  KAction *zoomInAct, *zoomOutAct, *backAct, *forwardAct,
    *startAct, *endAct, *endDocAct, *markAct, *gotoAct,
    *saveAction, *saveAsAction, *printAction, *readUpAct, *readDownAct;
  KAction *backAction;
  KAction *forwardAction;
  KAction *settingsAction;
  KAction* aboutAction;
  KToggleAction *watchAct,*useDocumentSpecifiedSize,
    *fitPageAct, *fitHeightAct, *fitWidthAct;
  KSelectAction* viewModeAction;

  KMultiPage *multiPage;
  KViewPartExtension *m_extension;

  bool pageChangeIsConnected;

  QWidget *mainWidget;

  pageSize  _paperSize;

  /** Some documents specify the paper size. In that case, the
      kmultipage implementation will call the slotPageSizeRequests()
      method. The method will then set this flag to 'true' and store
      the requested size in 'documentRequestedPaperSize'. */
  bool      documentHasRequestedPaperSize;
  pageSize  documentRequestedPaperSize;

  /** This entry stores the paper size that the user has requested in
      the preferences dialog. If that paper size is actually used or
      not, depends on if the document specifies a paper size of its
      own and if the user has chosen the option "use document
      specified paper size if available" */
  pageSize  userRequestedPaperSize;

  /** stores the current zoom value */
  class zoom      _zoomVal;
  pageSizeDialog *_pageSizeDialog;

  QTimer fitTimer;

  KAboutDialog* aboutDialog;
};


class KViewPartExtension : public KParts::BrowserExtension
{
  Q_OBJECT
  friend class KViewPart;

public:

  KViewPartExtension(KViewPart *parent);
  virtual ~KViewPartExtension() {}

};


#endif
